#include "StdAfx.h"
#include "WASAddGroup.h"
#include "ConvertUTF.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkAPIGlobal.h"


CWASAddGroup::CWASAddGroup(CWASActor *pActor) : 
	CWASCommand(pActor)
{
    m_nSortOrder = 0;
    m_addGroupType = buddyGroupAction_NotAdd;
}


CWASAddGroup::~CWASAddGroup(void)
{

}


void CWASAddGroup::ExecuteAsync()
{
	OutputDebugString( _T("CWASAddGroup::ExecuteAsync ===================================\n") );

	USES_UTF8_CONVERSION;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());

	char szSortOrder[10];
	if(pMessengerActor == NULL)
	{
		return;
	}
	_itoa(m_nSortOrder,szSortOrder,10);
	
	
	WSTRING strURL = m_pActor->GetWASServerAddress();
	strURL += _T("/ne/buddy/AddGroup.php");

	m_httpFacade.session().SetURL(W2U8(strURL.c_str()),	pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);

	m_httpFacade.session().AddBodyVariable("userid", W2U8(pMessengerActor->GetUserID()));
	m_httpFacade.session().AddBodyVariable("grpname", W2U8(m_strGroupName.c_str()));
	m_httpFacade.session().AddBodyVariable("ticket", pMessengerActor->GetMessengerTicket());
	//m_httpFacade.session().AddBodyVariable("sortorder", szSortOrder);

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	
	m_httpFacade.SendRequest(this);
}



bool CWASAddGroup::ParseAndEventFire(nfc::CIkXMLDom * pDom)
{
	if(pDom == NULL)
		return false;

	nfc::IIkXMLNode* pNode = pDom->GetdocumentElement()->selectSingleNode("nodes");
	if (pNode == NULL)
	{
		m_strWasErrMsg += _T("xml parse error : cannot find <nodes> tag");
		return false;
	}

	nfc::IIkXMLNodeList *pList = pNode->selectNodes("node");
	nfc::IIkXMLAttribute* pAttr;

	if( pList->Getlength() < 1 )
	{
		m_strWasErrMsg += _T("xml parse error : cannot find <nodes> tag");
		return false;
	}

	pNode = pList->Getitem(0);
	GT_ASSERT(pNode);
	pAttr = pNode->getAttribute("grpid");

	if( pAttr == NULL )
	{
		m_strWasErrMsg += _T("xml parse error : cannot find [grpid] attribute");
		return false;
	}

	m_sResult.nID = atoi(pAttr->Gettext().c_str());
	m_sResult.strName = m_strGroupName;

	pAttr = pNode->getAttribute("sortorder");

	if( pAttr == NULL )
	{
		m_strWasErrMsg += _T("xml parse error : cannot find [grpid] attribute");
		return false;
	}

	m_sResult.nSortOrder = atoi(pAttr->Gettext().c_str());

	return true;
}


// override in drived class
void CWASAddGroup::OnWasResponse(trid_t trid,
								  bool bSuccess,
								  int /*nWinINetErrorCode*/, 
								  int /*nHttpResponseCode*/, 
                                  LPCSTR /*szWasResCodeInHeader*/, 
                                  LPCSTR szWasResCodeInXml,
                                  nfc::CIkXMLDom * pDom)
{
	OutputDebugString( _T("CWASAddGroup::OnWasResponse ===================================\n") );

	if(bSuccess)
	{
		if(ParseAndEventFire(pDom) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);
	CGTObserved<IGTWASActorEventPrivate>::listobserver_t observers;
	m_pActor->GetObserversEx(&observers);
	CGTObserved<IGTWASActorEventPrivate>::listobserver_t::iterator it = observers.begin();

	GT_BUDDY_GROUP_DATA sData;
	m_sResult.Convert(sData);

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEventPrivate *pObserver = static_cast<IGTWASActorEventPrivate *>(*it);
		if(pObserver->OnGTWASActor_ResponseAddGroup(m_pActor, GetID(), m_eError, sData, m_addGroupType) == false)
			break;
	}
	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASAddGroup::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);
	CGTObserved<IGTWASActorEventPrivate>::listobserver_t observers;
	m_pActor->GetObserversEx(&observers);
	CGTObserved<IGTWASActorEventPrivate>::listobserver_t::iterator it = observers.begin();

	GT_BUDDY_GROUP_DATA sData;
	m_sResult.Convert(sData);

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEventPrivate *pObserver = static_cast<IGTWASActorEventPrivate *>(*it);
		if(pObserver->OnGTWASActor_ResponseAddGroup(m_pActor, GetID(), m_eError, sData, m_addGroupType) == false)
			break;
	}
	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}
